@@ -2,23 +2,52 @@ require "minitest/test"
 
 class Module # :nodoc:
   def infect_an_assertion meth, new_name, dont_flip = false # :nodoc:
+    block = dont_flip == :block
+    dont_flip = false if block
+    target_obj = block ? "_{obj.method}" : "_(obj)"
+
+    # https://eregon.me/blog/2021/02/13/correct-delegation-in-ruby-2-27-3.html
+    # Drop this when we can drop ruby 2.6 (aka after rails 6.1 EOL, ~2024-06)
+    kw_extra = "ruby2_keywords %p" % [new_name] if respond_to? :ruby2_keywords, true
+
     # warn "%-22p -> %p %p" % [meth, new_name, dont_flip]
-    self.class_eval <<-EOM
+    self.class_eval <<-EOM, __FILE__, __LINE__ + 1
       def #{new_name} *args
+        where = Minitest.filter_backtrace(caller).first
+        where = where.split(/:in /, 2).first # clean up noise
+        Kernel.warn "DEPRECATED: global use of #{new_name} from #\{where}. Use #{target_obj}.#{new_name} instead. This will fail in Minitest 6."
+        Minitest::Expectation.new(self, Minitest::Spec.current).#{new_name}(*args)
+      end
+      #{kw_extra}
+    EOM
+
+    Minitest::Expectation.class_eval <<-EOM, __FILE__, __LINE__ + 1
+      def #{new_name} *args
+        raise "Calling ##{new_name} outside of test." unless ctx
         case
         when #{!!dont_flip} then
-          Minitest::Spec.current.#{meth}(self, *args)
-        when Proc === self then
-          Minitest::Spec.current.#{meth}(*args, &self)
+          ctx.#{meth}(target, *args)
+        when #{block} && Proc === target then
+          ctx.#{meth}(*args, &target)
         else
-          Minitest::Spec.current.#{meth}(args.first, self, *args[1..-1])
+          ctx.#{meth}(args.first, target, *args[1..-1])
         end
       end
+      #{kw_extra}
     EOM
   end
 end
 
-module Kernel # :nodoc:
+# :stopdoc:
+module Minitest # fucking hell rdoc...
+  Expectation = Struct.new :target, :ctx
+end
+# :startdoc:
+
+##
+# Kernel extensions for minitest
+
+module Kernel
   ##
   # Describe a series of expectations for a given target +desc+.
   #
@@ -47,7 +76,7 @@ module Kernel # :nodoc:
   #
   # For some suggestions on how to improve your specs, try:
   #
-  # http://betterspecs.org
+  # https://betterspecs.org
   #
   # but do note that several items there are debatable or specific to
   # rspec.
@@ -56,14 +85,15 @@ module Kernel # :nodoc:
 
   def describe desc, *additional_desc, &block # :doc:
     stack = Minitest::Spec.describe_stack
-    name  = [stack.last, desc, *additional_desc].compact.join("::")
-    sclas = stack.last || if Class === self && is_a?(Minitest::Spec::DSL) then
-                            self
-                          else
-                            Minitest::Spec.spec_type desc, *additional_desc
-                          end
+    is_spec_class = Class === self && kind_of?(Minitest::Spec::DSL)
+    name  = [stack.last, desc, *additional_desc]
+    name.prepend self if stack.empty? && is_spec_class
+    sclas =
+      stack.last                 \
+      || (is_spec_class && self) \
+      || Minitest::Spec.spec_type(desc, *additional_desc)
 
-    cls = sclas.create name, desc
+    cls = sclas.create name.compact.join("::"), desc
 
     stack.push cls
     cls.class_eval(&block)
@@ -118,7 +148,7 @@ class Minitest::Spec < Minitest::Test
     #       desc.superclass == ActiveRecord::Base
     #     end
 
-    def register_spec_type(*args, &block)
+    def register_spec_type *args, &block
       if block then
         matcher, klass = block, args.first
       else
@@ -133,7 +163,7 @@ class Minitest::Spec < Minitest::Test
     #     spec_type("BlahController") # => Minitest::Spec::Rails
 
     def spec_type desc, *additional
-      TYPES.find { |matcher, klass|
+      TYPES.find { |matcher, _klass|
         if matcher.respond_to? :call then
           matcher.call desc, *additional
         else
@@ -146,10 +176,7 @@ class Minitest::Spec < Minitest::Test
       Thread.current[:describe_stack] ||= []
     end
 
-    ##
-    # Returns the children of this spec.
-
-    def children
+    def children # :nodoc:
       @children ||= []
     end
 
@@ -166,7 +193,7 @@ class Minitest::Spec < Minitest::Test
     #
     # Equivalent to Minitest::Test#setup.
 
-    def before type = nil, &block
+    def before _type = nil, &block
       define_method :setup do
         super()
         self.instance_eval(&block)
@@ -180,7 +207,7 @@ class Minitest::Spec < Minitest::Test
     #
     # Equivalent to Minitest::Test#teardown.
 
-    def after type = nil, &block
+    def after _type = nil, &block
       define_method :teardown do
         self.instance_eval(&block)
         super()
@@ -206,10 +233,12 @@ class Minitest::Spec < Minitest::Test
 
       name = "test_%04d_%s" % [ @specs, desc ]
 
+      undef_klasses = self.children.reject { |c| c.public_method_defined? name }
+
       define_method name, &block
 
-      self.children.each do |mod|
-        mod.send :undef_method, name if mod.public_method_defined? name
+      undef_klasses.each do |undef_klass|
+        undef_klass.send :undef_method, name
       end
 
       name
@@ -225,7 +254,7 @@ class Minitest::Spec < Minitest::Test
       pre, post = "let '#{name}' cannot ", ". Please use another name."
       methods = Minitest::Spec.instance_methods.map(&:to_s) - %w[subject]
       raise ArgumentError, "#{pre}begin with 'test'#{post}" if
-        name =~ /\Atest/
+        name.start_with? "test"
       raise ArgumentError, "#{pre}override a method in Minitest::Spec#{post}" if
         methods.include? name
 
@@ -244,7 +273,7 @@ class Minitest::Spec < Minitest::Test
     end
 
     def create name, desc # :nodoc:
-      cls = Class.new(self) do
+      cls = Class.new self do
         @name = name
         @desc = desc
 
@@ -260,26 +289,55 @@ class Minitest::Spec < Minitest::Test
       defined?(@name) ? @name : super
     end
 
-    def to_s # :nodoc:
-      name # Can't alias due to 1.8.7, not sure why
-    end
+    alias to_s name
 
-    # :stopdoc:
-    attr_reader :desc
-    alias :specify :it
+    attr_reader :desc # :nodoc:
+    alias specify it
+
+    ##
+    # Rdoc... why are you so dumb?
 
     module InstanceMethods
-      def before_setup
+      ##
+      # Takes a value or a block and returns a value monad that has
+      # all of Expectations methods available to it.
+      #
+      #   _(1 + 1).must_equal 2
+      #
+      # And for blocks:
+      #
+      #   _ { 1 + "1" }.must_raise TypeError
+      #
+      # This method of expectation-based testing is preferable to
+      # straight-expectation methods (on Object) because it stores its
+      # test context, bypassing our hacky use of thread-local variables.
+      #
+      # NOTE: At some point, the methods on Object will be deprecated
+      # and then removed.
+      #
+      # It is also aliased to #value and #expect for your aesthetic
+      # pleasure:
+      #
+      #         _(1 + 1).must_equal 2
+      #     value(1 + 1).must_equal 2
+      #    expect(1 + 1).must_equal 2
+
+      def _ value = nil, &block
+        Minitest::Expectation.new block || value, self
+      end
+
+      alias value _
+      alias expect _
+
+      def before_setup # :nodoc:
         super
         Thread.current[:current_spec] = self
       end
     end
 
-    def self.extended obj
+    def self.extended obj # :nodoc:
       obj.send :include, InstanceMethods
     end
-
-    # :startdoc:
   end
 
   extend DSL
