@@ -10,8 +10,9 @@ end
 
 class TestMinitestMock < Minitest::Test
   def setup
-    @mock = Minitest::Mock.new.expect(:foo, nil)
-    @mock.expect(:meaning_of_life, 42)
+    @mock = Minitest::Mock.new
+      .expect(:foo, nil)
+      .expect(:meaning_of_life, 42)
   end
 
   def test_create_stub_method
@@ -25,7 +26,7 @@ class TestMinitestMock < Minitest::Test
   def test_blow_up_if_not_called
     @mock.foo
 
-    util_verify_bad "expected meaning_of_life() => 42"
+    util_verify_bad "Expected meaning_of_life() => 42"
   end
 
   def test_not_blow_up_if_everything_called
@@ -36,22 +37,22 @@ class TestMinitestMock < Minitest::Test
   end
 
   def test_allow_expectations_to_be_added_after_creation
-    @mock.expect(:bar, true)
+    @mock.expect :bar, true
     assert @mock.bar
   end
 
   def test_not_verify_if_new_expected_method_is_not_called
     @mock.foo
     @mock.meaning_of_life
-    @mock.expect(:bar, true)
+    @mock.expect :bar, true
 
-    util_verify_bad "expected bar() => true"
+    util_verify_bad "Expected bar() => true"
   end
 
   def test_blow_up_on_wrong_number_of_arguments
     @mock.foo
     @mock.meaning_of_life
-    @mock.expect(:sum, 3, [1, 2])
+    @mock.expect :sum, 3, [1, 2]
 
     e = assert_raises ArgumentError do
       @mock.sum
@@ -63,7 +64,7 @@ class TestMinitestMock < Minitest::Test
   def test_return_mock_does_not_raise
     retval = Minitest::Mock.new
     mock = Minitest::Mock.new
-    mock.expect(:foo, retval)
+    mock.expect :foo, retval
     mock.foo
 
     assert_mock mock
@@ -72,8 +73,8 @@ class TestMinitestMock < Minitest::Test
   def test_mock_args_does_not_raise
     arg = Minitest::Mock.new
     mock = Minitest::Mock.new
-    mock.expect(:foo, nil, [arg])
-    mock.foo(arg)
+    mock.expect :foo, nil, [arg]
+    mock.foo arg
 
     assert_mock mock
   end
@@ -113,8 +114,6 @@ class TestMinitestMock < Minitest::Test
   end
 
   def test_expectations_can_be_satisfied_via_public_send
-    skip "Doesn't run on 1.8" if RUBY_VERSION < "1.9"
-
     @mock.public_send :foo
     @mock.public_send :meaning_of_life
 
@@ -124,10 +123,10 @@ class TestMinitestMock < Minitest::Test
   def test_blow_up_on_wrong_arguments
     @mock.foo
     @mock.meaning_of_life
-    @mock.expect(:sum, 3, [1, 2])
+    @mock.expect :sum, 3, [1, 2]
 
     e = assert_raises MockExpectationError do
-      @mock.sum(2, 4)
+      @mock.sum 2, 4
     end
 
     exp = "mocked method :sum called with unexpected arguments [2, 4]"
@@ -163,8 +162,8 @@ class TestMinitestMock < Minitest::Test
     a = Minitest::Mock.new
     b = Minitest::Mock.new
 
-    a.expect(:foo, :a)
-    b.expect(:foo, :b)
+    a.expect :foo, :a
+    b.expect :foo, :b
 
     assert_equal :a, a.foo
     assert_equal :b, b.foo
@@ -172,7 +171,7 @@ class TestMinitestMock < Minitest::Test
 
   def test_do_not_create_stub_method_on_new_mocks
     a = Minitest::Mock.new
-    a.expect(:foo, :a)
+    a.expect :foo, :a
 
     assert !Minitest::Mock.new.respond_to?(:foo)
   end
@@ -181,10 +180,77 @@ class TestMinitestMock < Minitest::Test
     @mock.expect :kind_of?, true, [String]
     @mock.expect :==, true, [1]
 
-    assert @mock.kind_of?(String), "didn't mock :kind_of\?"
+    assert @mock.kind_of?(String), "didn't mock :kind_of?"
     assert @mock == 1, "didn't mock :=="
   end
 
+  def test_assert_mock__pass
+    mock = Minitest::Mock.new
+    mock.expect :loose_expectation, true, [Integer]
+    mock.loose_expectation 1
+
+    result = assert_mock mock
+
+    assert_equal true, result
+  end
+
+  def assert_bad_mock klass, msg
+    mock = Minitest::Mock.new
+    mock.expect :foo, nil, [:bar]
+    mock.expect :foo, nil, [:baz]
+
+    mock.foo :bar
+
+    e = assert_raises klass do
+      yield mock
+    end
+
+    assert_equal msg, e.message
+  end
+
+  def test_verify__error
+    exp = "Expected foo(:baz) => nil, got [foo(:bar) => nil]"
+    assert_bad_mock MockExpectationError, exp do |mock|
+      mock.verify
+    end
+  end
+
+  def test_assert_mock__fail
+    exp = "Expected foo(:baz) => nil, got [foo(:bar) => nil]."
+    assert_bad_mock Minitest::Assertion, exp do |mock|
+      assert_mock mock
+    end
+  end
+
+  def test_assert_mock__fail_msg
+    exp = "BLAH.\nExpected foo(:baz) => nil, got [foo(:bar) => nil]."
+    assert_bad_mock Minitest::Assertion, exp do |mock|
+      assert_mock mock, "BLAH"
+    end
+  end
+
+  def test_assert_mock__fail_exp
+    exp = "Expected foo(:baz) => nil, got [foo(:bar) => nil]."
+    assert_bad_mock Minitest::Assertion, exp do |mock|
+      describe "X" do
+        it "y" do
+          _(mock).must_verify
+        end
+      end.new(:blah).send(:test_0001_y)
+    end
+  end
+
+  def test_assert_mock__fail_exp_msg
+    exp = "BLAH.\nExpected foo(:baz) => nil, got [foo(:bar) => nil]."
+    assert_bad_mock Minitest::Assertion, exp do |mock|
+      describe "X" do
+        it "y" do
+          _(mock).must_verify "BLAH"
+        end
+      end.new(:blah).send(:test_0001_y)
+    end
+  end
+
   def test_verify_allows_called_args_to_be_loosely_specified
     mock = Minitest::Mock.new
     mock.expect :loose_expectation, true, [Integer]
@@ -239,7 +305,7 @@ class TestMinitestMock < Minitest::Test
 
     e = assert_raises(MockExpectationError) { mock.verify }
 
-    exp = "expected foo(:baz) => nil, got [foo(:bar) => nil]"
+    exp = "Expected foo(:baz) => nil, got [foo(:bar) => nil]"
 
     assert_equal exp, e.message
   end
@@ -253,7 +319,7 @@ class TestMinitestMock < Minitest::Test
 
     e = assert_raises(MockExpectationError) { mock.verify }
 
-    exp = "expected foo(:bar) => nil, got [foo(:bar) => nil]"
+    exp = "Expected foo(:bar) => nil, got [foo(:bar) => nil]"
 
     assert_equal exp, e.message
   end
@@ -277,9 +343,10 @@ class TestMinitestMock < Minitest::Test
 
     e = assert_raises(MockExpectationError) { mock.verify }
 
-    exp = "expected foo(:kw=>false) => nil, got [foo(:kw=>true) => nil]"
+    exp = "Expected foo(%p) => nil, got [foo(%p) => nil]" \
+      % [{ :kw => false }, { :kw => true  }]
 
-    assert_equal exp, e.message
+    assert_equal exp.delete("{}"), e.message
   end
 
   def test_verify_passes_when_mock_block_returns_true
@@ -318,7 +385,7 @@ class TestMinitestMock < Minitest::Test
       k1 == arg1 && k2 == arg2 && k3 == arg3
     end
 
-    mock.foo(k1: arg1, k2: arg2, k3: arg3)
+    mock.foo k1: arg1, k2: arg2, k3: arg3
 
     assert_mock mock
   end
@@ -331,7 +398,7 @@ class TestMinitestMock < Minitest::Test
     end
 
     e = assert_raises ArgumentError do
-      mock.foo(k1: arg1, k2: arg2)
+      mock.foo k1: arg1, k2: arg2
     end
 
     # basically testing ruby ... need ? for ruby < 2.7 :(
@@ -346,7 +413,7 @@ class TestMinitestMock < Minitest::Test
     end
 
     e = assert_raises ArgumentError do
-      mock.foo(k1: arg1, k2: arg2, k3: arg3)
+      mock.foo k1: arg1, k2: arg2, k3: arg3
     end
 
     # basically testing ruby ... need ? for ruby < 2.7 :(
@@ -361,10 +428,12 @@ class TestMinitestMock < Minitest::Test
     end
 
     e = assert_raises MockExpectationError do
-      mock.foo(k1: arg1, k2: arg2, k3: :BAD!)
+      mock.foo k1: arg1, k2: arg2, k3: :BAD!
     end
 
-    exp = "mocked method :foo failed block w/ [] {:k1=>:bar, :k2=>[1, 2, 3], :k3=>:BAD!}"
+    exp = "mocked method :foo failed block w/ [] %p" \
+      % [{ :k1 => :bar, :k2 => [1, 2, 3], :k3 => :BAD! }]
+
     assert_equal exp, e.message
   end
 
@@ -373,7 +442,7 @@ class TestMinitestMock < Minitest::Test
     mock = Minitest::Mock.new
     mock.expect :foo, nil, k1: arg1, k2: arg2, k3: arg3
 
-    mock.foo(k1: arg1, k2: arg2, k3: arg3)
+    mock.foo k1: arg1, k2: arg2, k3: arg3
 
     assert_mock mock
   end
@@ -397,10 +466,10 @@ class TestMinitestMock < Minitest::Test
   def test_mock_block_is_passed_keyword_args__args__old_style_bad
     arg1, arg2, arg3 = :bar, [1, 2, 3], { :a => "a" }
     mock = Minitest::Mock.new
-    mock.expect :foo, nil, [{k1: arg1, k2: arg2, k3: arg3}]
+    mock.expect :foo, nil, [{ k1: arg1, k2: arg2, k3: arg3 }]
 
     e = assert_raises ArgumentError do
-      mock.foo(k1: arg1, k2: arg2, k3: arg3)
+      mock.foo k1: arg1, k2: arg2, k3: arg3
     end
 
     assert_equal "mocked method :foo expects 1 arguments, got []", e.message
@@ -410,9 +479,9 @@ class TestMinitestMock < Minitest::Test
     with_kwargs_env do
       arg1, arg2, arg3 = :bar, [1, 2, 3], { :a => "a" }
       mock = Minitest::Mock.new
-      mock.expect :foo, nil, [{k1: arg1, k2: arg2, k3: arg3}]
+      mock.expect :foo, nil, [{ k1: arg1, k2: arg2, k3: arg3 }]
 
-      mock.foo(k1: arg1, k2: arg2, k3: arg3)
+      mock.foo k1: arg1, k2: arg2, k3: arg3
 
       assert_mock mock
     end
@@ -423,10 +492,12 @@ class TestMinitestMock < Minitest::Test
       arg1, arg2, arg3 = :bar, [1, 2, 3], { :a => "a" }
       mock = Minitest::Mock.new
 
-      assert_output nil, /Using MT_KWARGS_HAC. yet passing kwargs/ do
+      assert_deprecation(/Using MT_KWARGS_HAC. yet passing kwargs/) do
         mock.expect :foo, nil, [{}], k1: arg1, k2: arg2, k3: arg3
       end
 
+      skip "-Werror" if error_on_warn? # mock above raised, so this is dead
+
       mock.foo({}, k1: arg1, k2: arg2, k3: arg3)
 
       assert_mock mock
@@ -439,10 +510,10 @@ class TestMinitestMock < Minitest::Test
     mock.expect :foo, nil, k1: arg1, k2: arg2, k3: arg3
 
     e = assert_raises ArgumentError do
-      mock.foo(k1: arg1, k2: arg2)
+      mock.foo k1: arg1, k2: arg2
     end
 
-    assert_equal "mocked method :foo expects 3 keyword arguments, got %p" % {k1: arg1, k2: arg2}, e.message
+    assert_equal "mocked method :foo expects 3 keyword arguments, got %p" % { k1: arg1, k2: arg2 }, e.message
   end
 
   def test_mock_block_is_passed_keyword_args__args_bad_extra
@@ -451,10 +522,10 @@ class TestMinitestMock < Minitest::Test
     mock.expect :foo, nil, k1: arg1, k2: arg2
 
     e = assert_raises ArgumentError do
-      mock.foo(k1: arg1, k2: arg2, k3: arg3)
+      mock.foo k1: arg1, k2: arg2, k3: arg3
     end
 
-    assert_equal "mocked method :foo expects 2 keyword arguments, got %p" % {k1: arg1, k2: arg2, k3: arg3}, e.message
+    assert_equal "mocked method :foo expects 2 keyword arguments, got %p" % { k1: arg1, k2: arg2, k3: arg3 }, e.message
   end
 
   def test_mock_block_is_passed_keyword_args__args_bad_key
@@ -463,7 +534,7 @@ class TestMinitestMock < Minitest::Test
     mock.expect :foo, nil, k1: arg1, k2: arg2, k3: arg3
 
     e = assert_raises MockExpectationError do
-      mock.foo(k1: arg1, k2: arg2, BAD: arg3)
+      mock.foo k1: arg1, k2: arg2, BAD: arg3
     end
 
     assert_includes e.message, "unexpected keywords [:k1, :k2, :k3]"
@@ -476,18 +547,18 @@ class TestMinitestMock < Minitest::Test
     mock.expect :foo, nil, k1: arg1, k2: arg2, k3: arg3
 
     e = assert_raises MockExpectationError do
-      mock.foo(k1: arg1, k2: :BAD!, k3: arg3)
+      mock.foo k1: arg1, k2: :BAD!, k3: arg3
     end
 
-    assert_match(/unexpected keyword arguments.* vs .*:k2=>:BAD!/, e.message)
+    bad = { :k2 => :BAD! }.inspect.delete "{}"
+    assert_match(/unexpected keyword arguments.* vs .*#{bad}/, e.message)
   end
 
   def test_mock_block_is_passed_function_block
     mock = Minitest::Mock.new
     block = proc { "bar" }
     mock.expect :foo, nil do |arg, &blk|
-      arg == "foo" &&
-      blk == block
+      arg == "foo" && blk == block
     end
     mock.foo "foo", &block
     assert_mock mock
@@ -495,8 +566,8 @@ class TestMinitestMock < Minitest::Test
 
   def test_mock_forward_keyword_arguments
     mock = Minitest::Mock.new
-    mock.expect(:foo, nil) { |bar:| bar == 'bar' }
-    mock.foo(bar: 'bar')
+    mock.expect(:foo, nil) { |bar:| bar == "bar" }
+    mock.foo bar: "bar"
     assert_mock mock
   end
 
@@ -515,7 +586,7 @@ class TestMinitestMock < Minitest::Test
   def test_mock_block_raises_if_args_passed
     mock = Minitest::Mock.new
 
-    e = assert_raises(ArgumentError) do
+    e = assert_raises ArgumentError do
       mock.expect :foo, nil, [:a, :b, :c] do
         true
       end
@@ -529,8 +600,8 @@ class TestMinitestMock < Minitest::Test
   def test_mock_block_raises_if_kwargs_passed
     mock = Minitest::Mock.new
 
-    e = assert_raises(ArgumentError) do
-      mock.expect :foo, nil, kwargs:1 do
+    e = assert_raises ArgumentError do
+      mock.expect :foo, nil, kwargs: 1 do
         true
       end
     end
@@ -542,7 +613,7 @@ class TestMinitestMock < Minitest::Test
 
   def test_mock_returns_retval_when_called_with_block
     mock = Minitest::Mock.new
-    mock.expect(:foo, 32) do
+    mock.expect :foo, 32 do
       true
     end
 
@@ -561,7 +632,7 @@ class TestMinitestMock < Minitest::Test
 
   def test_mock_called_via_send
     mock = Minitest::Mock.new
-    mock.expect(:foo, true)
+    mock.expect :foo, true
 
     mock.send :foo
     assert_mock mock
@@ -569,7 +640,7 @@ class TestMinitestMock < Minitest::Test
 
   def test_mock_called_via___send__
     mock = Minitest::Mock.new
-    mock.expect(:foo, true)
+    mock.expect :foo, true
 
     mock.__send__ :foo
     assert_mock mock
@@ -577,9 +648,9 @@ class TestMinitestMock < Minitest::Test
 
   def test_mock_called_via_send_with_args
     mock = Minitest::Mock.new
-    mock.expect(:foo, true, [1, 2, 3])
+    mock.expect :foo, true, [1, 2, 3]
 
-    mock.send(:foo, 1, 2, 3)
+    mock.send :foo, 1, 2, 3
     assert_mock mock
   end
 
@@ -655,7 +726,7 @@ class TestMinitestStub < Minitest::Test
     end
   end
 
-  def test_stub_value
+  def test_stub_value__literal
     assert_stub 42
   end
 
@@ -686,7 +757,7 @@ class TestMinitestStub < Minitest::Test
   end
 
   def test_stub_yield_self
-    obj = "foo"
+    obj = +"foo"
 
     val = obj.stub :to_s, "bar" do |s|
       s.to_s
@@ -712,7 +783,7 @@ class TestMinitestStub < Minitest::Test
       end
     end
 
-    val = dynamic.stub(:found, true) do |s|
+    val = dynamic.stub :found, true do |s|
       s.found
     end
 
@@ -727,14 +798,17 @@ class TestMinitestStub < Minitest::Test
       end
     end
 
-    exp = jruby? ? /Undefined method nope_nope_nope for '#{self.class}::Time'/ :
-      /undefined method `nope_nope_nope' for( class)? `#{self.class}::Time'/
+    exp = if jruby? then
+            /Undefined method nope_nope_nope for '#{self.class}::Time'/
+          else
+            /undefined method [`']nope_nope_nope' for( class)? [`']#{self.class}::Time'/
+          end
     assert_match exp, e.message
   end
 
   def test_mock_with_yield
     mock = Minitest::Mock.new
-    mock.expect(:write, true) do
+    mock.expect :write, true do
       true
     end
     rs = nil
@@ -751,7 +825,7 @@ class TestMinitestStub < Minitest::Test
     mock = Minitest::Mock.new
     rs = nil
 
-    File.stub :open, true, mock, kw:42 do
+    File.stub :open, true, mock, kw: 42 do
       File.open "foo.txt", "r" do |f, kw:|
         rs = kw
       end
@@ -760,8 +834,6 @@ class TestMinitestStub < Minitest::Test
     @tc.assert_equal 42, rs
   end
 
-  alias test_stub_value__old test_stub_value # TODO: remove/rename
-
   ## Permutation Sets:
 
   # [:value, :lambda]
@@ -816,7 +888,7 @@ class TestMinitestStub < Minitest::Test
   # [:value,  :block_call, :args] =>  N/A
 
   class Bar
-    def call
+    def call &_ # to ignore unused block
       puts "hi"
     end
   end
@@ -834,7 +906,7 @@ class TestMinitestStub < Minitest::Test
   end
 
   class Keywords
-    def self.args req, kw1:, kw2:24
+    def self.args req, kw1:, kw2: 24
       [req, kw1, kw2]
     end
   end
@@ -848,7 +920,7 @@ class TestMinitestStub < Minitest::Test
   def test_stub__hash_as_last_real_arg
     with_kwargs_env do
       token = Object.new
-      def token.create_with_retry u, p; raise "shouldn't see this"; end
+      def token.create_with_retry _u, _p; raise "shouldn't see this"; end
 
       controller = Object.new
       controller.define_singleton_method :create do |u, p|
@@ -956,7 +1028,7 @@ class TestMinitestStub < Minitest::Test
   def test_stub_lambda_block_call_5
     @assertion_count += 1
     rs = nil
-    io = StringIO.new "", "w"
+    io = StringIO.new(+"", "w")
     File.stub5 :open, lambda { |p, m, &blk| blk and blk.call io } do
       File.open "foo.txt", "r" do |f|
         rs = f && f.write("woot")
@@ -971,10 +1043,10 @@ class TestMinitestStub < Minitest::Test
 
     @assertion_count += 1
     rs = nil
-    io = StringIO.new "", "w"
+    io = StringIO.new(+"", "w")
     File.stub6 :open, lambda { |p, m, &blk| blk.call io } do
       File.open "foo.txt", "r" do |f|
-        rs = f.write("woot")
+        rs = f.write "woot"
       end
     end
     @tc.assert_equal 4, rs
@@ -984,10 +1056,10 @@ class TestMinitestStub < Minitest::Test
   def test_stub_lambda_block_call_args_5
     @assertion_count += 1
     rs = nil
-    io = StringIO.new "", "w"
+    io = StringIO.new(+"", "w")
     File.stub5(:open, lambda { |p, m, &blk| blk and blk.call io }, :WTF?) do
       File.open "foo.txt", "r" do |f|
-        rs = f.write("woot")
+        rs = f.write "woot"
       end
     end
     @tc.assert_equal 4, rs
@@ -999,10 +1071,10 @@ class TestMinitestStub < Minitest::Test
 
     @assertion_count += 1
     rs = nil
-    io = StringIO.new "", "w"
+    io = StringIO.new(+"", "w")
     File.stub6(:open, lambda { |p, m, &blk| blk.call io }, :WTF?) do
       File.open "foo.txt", "r" do |f|
-        rs = f.write("woot")
+        rs = f.write "woot"
       end
     end
     @tc.assert_equal 4, rs
@@ -1014,11 +1086,11 @@ class TestMinitestStub < Minitest::Test
 
     @assertion_count += 2
     rs = nil
-    io = StringIO.new "", "w"
+    io = StringIO.new(+"", "w")
     @tc.assert_raises ArgumentError do
       File.stub6_2(:open, lambda { |p, m, &blk| blk.call io }, :WTF?) do
         File.open "foo.txt", "r" do |f|
-          rs = f.write("woot")
+          rs = f.write "woot"
         end
       end
     end
@@ -1064,10 +1136,10 @@ class TestMinitestStub < Minitest::Test
   def test_stub_value_block_args_5
     @assertion_count += 2
     rs = nil
-    io = StringIO.new "", "w"
+    io = StringIO.new(+"", "w")
     File.stub5 :open, :value, io do
       result = File.open "foo.txt", "r" do |f|
-        rs = f.write("woot")
+        rs = f.write "woot"
       end
       @tc.assert_equal :value, result
     end
@@ -1083,7 +1155,7 @@ class TestMinitestStub < Minitest::Test
         end
       end
     end
-    exp = "undefined method `write' for nil:NilClass"
+    exp = /undefined method [`']write' for nil/
     assert_match exp, e.message
   end
 
@@ -1092,11 +1164,11 @@ class TestMinitestStub < Minitest::Test
 
     @assertion_count += 2
     rs = nil
-    io = StringIO.new "", "w"
+    io = StringIO.new(+"", "w")
     assert_deprecated do
       File.stub6 :open, :value, io do
         result = File.open "foo.txt", "r" do |f|
-          rs = f.write("woot")
+          rs = f.write "woot"
         end
         @tc.assert_equal :value, result
       end
@@ -1110,7 +1182,7 @@ class TestMinitestStub < Minitest::Test
 
     @assertion_count += 2
     rs = nil
-    io = StringIO.new "", "w"
+    io = StringIO.new(+"", "w")
     @tc.assert_raises ArgumentError do
       File.stub6_2 :open, :value, io do
         result = File.open "foo.txt", "r" do |f|
