@@ -1,4 +1,3 @@
-# encoding: utf-8
 require "minitest/metametameta"
 require "stringio"
 
@@ -12,15 +11,12 @@ class ExampleA; end
 class ExampleB < ExampleA; end
 
 describe Minitest::Spec do
-  # helps to deal with 2.4 deprecation of Fixnum for Integer
-  Int = 1.class
-
   # do not parallelize this suite... it just can"t handle it.
 
   def assert_triggered expected = "blah", klass = Minitest::Assertion
     @assertion_count += 1
 
-    e = assert_raises(klass) do
+    e = assert_raises klass do
       yield
     end
 
@@ -29,17 +25,17 @@ describe Minitest::Spec do
     msg.gsub!(/(\d\.\d{6})\d+/, '\1xxx') # normalize: ruby version, impl, platform
     msg.gsub!(/:0x[Xa-fA-F0-9]{4,}[ @].+?>/, ":0xXXXXXX@PATH>")
 
-    if expected
+    return unless expected
+
+    @assertion_count += 1
+    case expected
+    when String then
+      assert_equal expected, msg
+    when Regexp then
       @assertion_count += 1
-      case expected
-      when String then
-        assert_equal expected, msg
-      when Regexp then
-        @assertion_count += 1
-        assert_match expected, msg
-      else
-        flunk "Unknown: #{expected.inspect}"
-      end
+      assert_match expected, msg
+    else
+      flunk "Unknown: #{expected.inspect}"
     end
   end
 
@@ -112,7 +108,7 @@ describe Minitest::Spec do
   end
 
   it "needs to catch an expected exception" do
-    @assertion_count = 2
+    @assertion_count -= 2
 
     expect { raise "blah" }.must_raise RuntimeError
     expect { raise Minitest::Assertion }.must_raise Minitest::Assertion
@@ -137,6 +133,46 @@ describe Minitest::Spec do
     end
   end
 
+  def good_pattern
+    capture_io do # 3.0 is noisy
+      eval "[1,2,3] => [Integer, Integer, Integer]" # eval to escape parser for ruby<3
+    end
+  end
+
+  def bad_pattern
+    capture_io do # 3.0 is noisy
+      eval "[1,2,3] => [Integer, Integer]" # eval to escape parser for ruby<3
+    end
+  end
+
+  it "needs to pattern match" do
+    @assertion_count = 1
+
+    if RUBY_VERSION > "3" then
+      expect { good_pattern }.must_pattern_match
+    else
+      assert_raises NotImplementedError do
+        expect {}.must_pattern_match
+      end
+    end
+  end
+
+  it "needs to error on bad pattern match" do
+    skip unless RUBY_VERSION > "3"
+
+    @assertion_count = 1
+
+    exp = if RUBY_VERSION.start_with? "3.0"
+            "[1, 2, 3]" # terrible error message!
+          else
+            /length mismatch/
+          end
+
+    assert_triggered exp do
+      expect { bad_pattern }.must_pattern_match
+    end
+  end
+
   it "needs to ensure silence" do
     @assertion_count -= 1 # no msg
     @assertion_count += 2 # assert_output is 2 assertions
@@ -156,7 +192,7 @@ describe Minitest::Spec do
     methods = Minitest::Expectations.public_instance_methods.grep(/must|wont/)
     methods.map!(&:to_s) if Symbol === methods.first
 
-    musts, wonts = methods.sort.partition { |m| m =~ /must/ }
+    musts, wonts = methods.sort.partition { |m| m.include? "must" }
 
     expected_musts = %w[must_be
                         must_be_close_to
@@ -172,14 +208,16 @@ describe Minitest::Spec do
                         must_include
                         must_match
                         must_output
+                        must_pattern_match
                         must_raise
                         must_respond_to
                         must_throw
+                        must_verify
                         path_must_exist]
 
-    bad = %w[not raise throw send output be_silent]
+    bad = %w[not raise throw send output be_silent verify]
 
-    expected_wonts = expected_musts.map { |m| m.sub(/must/, "wont") }.sort
+    expected_wonts = expected_musts.map { |m| m.sub("must", "wont") }.sort
     expected_wonts.reject! { |m| m =~ /wont_#{Regexp.union(*bad)}/ }
 
     _(musts).must_equal expected_musts
@@ -243,18 +281,14 @@ describe Minitest::Spec do
   end
 
   it "needs to warn on equality with nil" do
-    @assertion_count += 1 # extra test
+    @assertion_count = 3
+    @assertion_count += 2 unless error_on_warn? # 2 extra assertions
 
-    out, err = capture_io do
+    exp = /DEPRECATED: Use assert_nil if expecting nil from .* This will fail in Minitest 6./
+
+    assert_deprecation exp do
       assert_success _(nil).must_equal(nil)
     end
-
-    exp = "DEPRECATED: Use assert_nil if expecting nil from #{__FILE__}:#{__LINE__-3}. " \
-      "This will fail in Minitest 6.\n"
-    exp = "" if $-w.nil?
-
-    assert_empty out
-    assert_equal exp, err
   end
 
   it "needs to verify floats outside a delta" do
@@ -362,12 +396,12 @@ describe Minitest::Spec do
   it "needs to verify instances of a class" do
     assert_success _(42).wont_be_instance_of(String)
 
-    assert_triggered "Expected 42 to not be a kind of #{Int.name}." do
-      _(42).wont_be_kind_of Int
+    assert_triggered "Expected 42 to not be a kind of Integer." do
+      _(42).wont_be_kind_of Integer
     end
 
-    assert_triggered "msg.\nExpected 42 to not be an instance of #{Int.name}." do
-      _(42).wont_be_instance_of Int, "msg"
+    assert_triggered "msg.\nExpected 42 to not be an instance of Integer." do
+      _(42).wont_be_instance_of Integer, "msg"
     end
   end
 
@@ -377,26 +411,26 @@ describe Minitest::Spec do
     assert_success _(42).wont_be_kind_of(String)
     assert_success _(proc {}).wont_be_kind_of(String)
 
-    assert_triggered "Expected 42 to not be a kind of #{Int.name}." do
-      _(42).wont_be_kind_of Int
+    assert_triggered "Expected 42 to not be a kind of Integer." do
+      _(42).wont_be_kind_of Integer
     end
 
-    assert_triggered "msg.\nExpected 42 to not be a kind of #{Int.name}." do
-      _(42).wont_be_kind_of Int, "msg"
+    assert_triggered "msg.\nExpected 42 to not be a kind of Integer." do
+      _(42).wont_be_kind_of Integer, "msg"
     end
   end
 
   it "needs to verify kinds of objects" do
     @assertion_count += 3 # extra test
 
-    assert_success _(6 * 7).must_be_kind_of(Int)
+    assert_success _(6 * 7).must_be_kind_of(Integer)
     assert_success _(6 * 7).must_be_kind_of(Numeric)
 
-    assert_triggered "Expected 42 to be a kind of String, not #{Int.name}." do
+    assert_triggered "Expected 42 to be a kind of String, not Integer." do
       _(6 * 7).must_be_kind_of String
     end
 
-    assert_triggered "msg.\nExpected 42 to be a kind of String, not #{Int.name}." do
+    assert_triggered "msg.\nExpected 42 to be a kind of String, not Integer." do
       _(6 * 7).must_be_kind_of String, "msg"
     end
 
@@ -535,7 +569,8 @@ describe Minitest::Spec do
 
     it "can NOT use must_equal in a thread. It must use expect in a thread" do
       skip "N/A" if ENV["MT_NO_EXPECTATIONS"]
-      assert_raises RuntimeError do
+
+      assert_raises RuntimeError, Minitest::UnexpectedWarning do
         capture_io do
           Thread.new { (1 + 1).must_equal 2 }.join
         end
@@ -545,29 +580,33 @@ describe Minitest::Spec do
     it "fails gracefully when expectation used outside of `it`" do
       skip "N/A" if ENV["MT_NO_EXPECTATIONS"]
 
-      @assertion_count += 1
+      @assertion_count += 2 # assert_match is compound
 
-      e = assert_raises RuntimeError do
+      e = assert_raises RuntimeError, Minitest::UnexpectedWarning do
         capture_io do
           Thread.new { # forces ctx to be nil
-            describe("woot") do
+            describe "woot" do
               (1 + 1).must_equal 2
             end
           }.join
         end
       end
 
-      assert_equal "Calling #must_equal outside of test.", e.message
+      exp = "Calling #must_equal outside of test."
+      exp = "DEPRECATED: global use of must_equal from" if error_on_warn?
+
+      assert_match exp, e.message
     end
 
     it "deprecates expectation used without _" do
       skip "N/A" if ENV["MT_NO_EXPECTATIONS"]
 
-      @assertion_count += 3
+      @assertion_count += 1
+      @assertion_count += 2 unless error_on_warn?
 
       exp = /DEPRECATED: global use of must_equal from/
 
-      assert_output "", exp do
+      assert_deprecation exp do
         (1 + 1).must_equal 2
       end
     end
@@ -577,12 +616,13 @@ describe Minitest::Spec do
     it "deprecates expectation used without _ with empty backtrace_filter" do
       skip "N/A" if ENV["MT_NO_EXPECTATIONS"]
 
-      @assertion_count += 3
+      @assertion_count += 1
+      @assertion_count += 2 unless error_on_warn?
 
       exp = /DEPRECATED: global use of must_equal from/
 
       with_empty_backtrace_filter do
-        assert_output "", exp do
+        assert_deprecation exp do
           (1 + 1).must_equal 2
         end
       end
@@ -613,9 +653,9 @@ describe Minitest::Spec do
   end
 
   it "needs to verify types of objects" do
-    assert_success _(6 * 7).must_be_instance_of(Int)
+    assert_success _(6 * 7).must_be_instance_of(Integer)
 
-    exp = "Expected 42 to be an instance of String, not #{Int.name}."
+    exp = "Expected 42 to be an instance of String, not Integer."
 
     assert_triggered exp do
       _(6 * 7).must_be_instance_of String
@@ -642,7 +682,7 @@ describe Minitest::Spec do
     assert_success _(41).must_be(:<, 42)
 
     assert_triggered "Expected 42 to be < 41." do
-      _(42).must_be(:<, 41)
+      _(42).must_be :<, 41
     end
   end
 
@@ -659,19 +699,17 @@ describe Minitest::Spec do
   it "needs to verify using respond_to" do
     assert_success _(42).must_respond_to(:+)
 
-    assert_triggered "Expected 42 (#{Int.name}) to respond to #clear." do
+    assert_triggered "Expected 42 (Integer) to respond to #clear." do
       _(42).must_respond_to :clear
     end
 
-    assert_triggered "msg.\nExpected 42 (#{Int.name}) to respond to #clear." do
+    assert_triggered "msg.\nExpected 42 (Integer) to respond to #clear." do
       _(42).must_respond_to :clear, "msg"
     end
   end
 end
 
 describe Minitest::Spec, :let do
-  i_suck_and_my_tests_are_order_dependent!
-
   def _count
     $let_count ||= 0
   end
@@ -682,21 +720,21 @@ describe Minitest::Spec, :let do
   end
 
   it "is evaluated once per example" do
-    _(_count).must_equal 0
+    exp = _count + 1
 
-    _(count).must_equal 1
-    _(count).must_equal 1
+    _(count).must_equal exp
+    _(count).must_equal exp
 
-    _(_count).must_equal 1
+    _(_count).must_equal exp
   end
 
   it "is REALLY evaluated once per example" do
-    _(_count).must_equal 1
+    exp = _count + 1
 
-    _(count).must_equal 2
-    _(count).must_equal 2
+    _(count).must_equal exp
+    _(count).must_equal exp
 
-    _(_count).must_equal 2
+    _(_count).must_equal exp
   end
 
   it 'raises an error if the name begins with "test"' do
@@ -710,9 +748,9 @@ describe Minitest::Spec, :let do
   it "doesn't raise an error if it is just another let" do
     v = proc do
       describe :outer do
-        let(:bar)
+        let :bar
         describe :inner do
-          let(:bar)
+          let :bar
         end
       end
       :good
@@ -913,6 +951,23 @@ class TestMeta < MetaMetaMetaTestCase
     assert_equal "ExampleB::random_method", spec_b.name
   end
 
+  def test_name_inside_class
+    spec_a = nil
+    spec_b = nil
+    inside_class_example = Class.new Minitest::Spec
+    Object.const_set :InsideClassExample, inside_class_example
+    inside_class_example.class_eval do
+      spec_a = describe "a" do
+        spec_b = describe "b" do; end
+      end
+    end
+
+    assert_equal "InsideClassExample::a", spec_a.name
+    assert_equal "InsideClassExample::a::b", spec_b.name
+  ensure
+    Object.send :remove_const, :InsideClassExample
+  end
+
   def test_structure
     x, y, z, * = util_structure
 
@@ -975,8 +1030,9 @@ class TestMeta < MetaMetaMetaTestCase
       z = describe "second thingy" do end
     end
 
-    test_methods = ["test_0001_top level it",
-                    "test_0002_не латинские &いった α, β, γ, δ, ε hello!!! world",
+    test_methods = [
+                     "test_0001_top level it",
+                     "test_0002_не латинские &いった α, β, γ, δ, ε hello!!! world",
                    ].sort
 
     assert_equal test_methods, [x1, x2]
