@@ -1,21 +1,18 @@
-# encoding: UTF-8
-
-require "pathname"
 require "minitest/metametameta"
 
-if defined? Encoding then
-  e = Encoding.default_external
-  if e != Encoding::UTF_8 then
-    warn ""
-    warn ""
-    warn "NOTE: External encoding #{e} is not UTF-8. Tests WILL fail."
-    warn "      Run tests with `RUBYOPT=-Eutf-8 rake` to avoid errors."
-    warn ""
-    warn ""
-  end
+e = Encoding.default_external
+if e != Encoding::UTF_8 then
+  warn ""
+  warn ""
+  warn "NOTE: External encoding #{e} is not UTF-8. Tests WILL fail."
+  warn "      Run tests with `RUBYOPT=-Eutf-8 rake` to avoid errors."
+  warn ""
+  warn ""
 end
 
 class Minitest::Runnable
+  attr_reader :gc_stats # only needed if running w/ minitest-gcstats
+
   def whatever # faked for testing
     assert true
   end
@@ -24,48 +21,45 @@ end
 class TestMinitestUnit < MetaMetaMetaTestCase
   parallelize_me!
 
-  pwd = Pathname.new File.expand_path Dir.pwd
-  basedir = Pathname.new(File.expand_path "lib/minitest") + "mini"
-  basedir = basedir.relative_path_from(pwd).to_s
-  MINITEST_BASE_DIR = basedir[/\A\./] ? basedir : "./#{basedir}"
-  BT_MIDDLE = ["#{MINITEST_BASE_DIR}/test.rb:161:in `each'",
-               "#{MINITEST_BASE_DIR}/test.rb:158:in `each'",
-               "#{MINITEST_BASE_DIR}/test.rb:139:in `run'",
-               "#{MINITEST_BASE_DIR}/test.rb:106:in `run'"]
+  MINITEST_BASE_DIR = "./lib/minitest/mini"
+  BT_MIDDLE = ["#{MINITEST_BASE_DIR}/test.rb:161:in 'each'",
+               "#{MINITEST_BASE_DIR}/test.rb:158:in 'each'",
+               "#{MINITEST_BASE_DIR}/test.rb:139:in 'run'",
+               "#{MINITEST_BASE_DIR}/test.rb:106:in 'run'"]
 
   def test_filter_backtrace
     # this is a semi-lame mix of relative paths.
     # I cheated by making the autotest parts not have ./
-    bt = (["lib/autotest.rb:571:in `add_exception'",
-           "test/test_autotest.rb:62:in `test_add_exception'",
-           "#{MINITEST_BASE_DIR}/test.rb:165:in `__send__'"] +
+    bt = (["lib/autotest.rb:571:in 'add_exception'",
+           "test/test_autotest.rb:62:in 'test_add_exception'",
+           "#{MINITEST_BASE_DIR}/test.rb:165:in '__send__'"] +
           BT_MIDDLE +
           ["#{MINITEST_BASE_DIR}/test.rb:29",
            "test/test_autotest.rb:422"])
     bt = util_expand_bt bt
 
-    ex = ["lib/autotest.rb:571:in `add_exception'",
-          "test/test_autotest.rb:62:in `test_add_exception'"]
+    ex = ["lib/autotest.rb:571:in 'add_exception'",
+          "test/test_autotest.rb:62:in 'test_add_exception'"]
     ex = util_expand_bt ex
 
     Minitest::Test.io_lock.synchronize do # try not to trounce in parallel
-      fu = Minitest.filter_backtrace(bt)
+      fu = Minitest.filter_backtrace bt
 
       assert_equal ex, fu
     end
   end
 
   def test_filter_backtrace_all_unit
-    bt = (["#{MINITEST_BASE_DIR}/test.rb:165:in `__send__'"] +
+    bt = (["#{MINITEST_BASE_DIR}/test.rb:165:in '__send__'"] +
           BT_MIDDLE +
           ["#{MINITEST_BASE_DIR}/test.rb:29"])
     ex = bt.clone
-    fu = Minitest.filter_backtrace(bt)
+    fu = Minitest.filter_backtrace bt
     assert_equal ex, fu
   end
 
   def test_filter_backtrace_unit_starts
-    bt = (["#{MINITEST_BASE_DIR}/test.rb:165:in `__send__'"] +
+    bt = (["#{MINITEST_BASE_DIR}/test.rb:165:in '__send__'"] +
           BT_MIDDLE +
           ["#{MINITEST_BASE_DIR}/mini/test.rb:29",
            "-e:1"])
@@ -94,24 +88,24 @@ class TestMinitestUnit < MetaMetaMetaTestCase
       end
 
       def test_this_is_non_ascii_failure_message
-        fail 'ЁЁЁ'.force_encoding('ASCII-8BIT')
+        raise "ЁЁЁ".dup.force_encoding(Encoding::BINARY)
       end
     end
 
-    expected = clean <<-EOM
+    expected = <<~EOM
       FE
 
       Finished in 0.00
 
         1) Failure:
       FakeNamedTestXX#test_this_is_not_ascii_assertion [FILE:LINE]:
-      Expected: \"ЁЁЁ\"
-        Actual: \"ёёё\"
+      Expected: "ЁЁЁ"
+        Actual: "ёёё"
 
         2) Error:
       FakeNamedTestXX#test_this_is_non_ascii_failure_message:
       RuntimeError: ЁЁЁ
-          FILE:LINE:in `test_this_is_non_ascii_failure_message'
+          FILE:LINE:in 'test_this_is_non_ascii_failure_message'
 
       2 runs, 1 assertions, 1 failures, 1 errors, 0 skips
     EOM
@@ -163,8 +157,26 @@ class TestMinitestUnit < MetaMetaMetaTestCase
     refute_predicate test, :skipped?
   end
 
+  def test_skipped_is_reachable
+    test_class = Class.new FakeNamedTest do
+      def test_omg
+        skip
+      ensure
+        flunk unless skipped?
+      end
+    end
+
+    test = test_class.new :test_omg
+    test.run
+
+    refute_predicate test, :error?
+    refute_predicate test, :passed?
+
+    assert_predicate test, :skipped?
+  end
+
   def util_expand_bt bt
-    bt.map { |f| (f =~ /^\./) ? File.expand_path(f) : f }
+    bt.map { |f| f.start_with?(".") ? File.expand_path(f) : f }
   end
 end
 
@@ -208,15 +220,14 @@ class TestMinitestRunner < MetaMetaMetaTestCase
   def test_class_runnables
     @assertion_count = 0
 
-    tc = Class.new(Minitest::Test)
+    tc = Class.new Minitest::Test
 
     assert_equal 1, Minitest::Test.runnables.size
     assert_equal [tc], Minitest::Test.runnables
   end
 
   def test_run_test
-    @tu =
-    Class.new FakeNamedTest do
+    @tu = Class.new FakeNamedTest do
       attr_reader :foo
 
       def run
@@ -232,7 +243,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
       end
     end
 
-    expected = clean <<-EOM
+    expected = <<~EOM
       .
 
       Finished in 0.00
@@ -244,8 +255,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
   end
 
   def test_run_error
-    @tu =
-    Class.new FakeNamedTest do
+    @tu = Class.new FakeNamedTest do
       def test_something
         assert true
       end
@@ -255,7 +265,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
       end
     end
 
-    expected = clean <<-EOM
+    expected = <<~EOM
       .E
 
       Finished in 0.00
@@ -263,7 +273,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
         1) Error:
       FakeNamedTestXX#test_error:
       RuntimeError: unhandled exception
-          FILE:LINE:in \`test_error\'
+          FILE:LINE:in 'test_error'
 
       2 runs, 1 assertions, 0 failures, 1 errors, 0 skips
     EOM
@@ -272,8 +282,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
   end
 
   def test_run_error_teardown
-    @tu =
-    Class.new FakeNamedTest do
+    @tu = Class.new FakeNamedTest do
       def test_something
         assert true
       end
@@ -283,7 +292,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
       end
     end
 
-    expected = clean <<-EOM
+    expected = <<~EOM
       E
 
       Finished in 0.00
@@ -291,7 +300,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
         1) Error:
       FakeNamedTestXX#test_something:
       RuntimeError: unhandled exception
-          FILE:LINE:in \`teardown\'
+          FILE:LINE:in 'teardown'
 
       1 runs, 1 assertions, 0 failures, 1 errors, 0 skips
     EOM
@@ -302,7 +311,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
   def test_run_failing
     setup_basic_tu
 
-    expected = clean <<-EOM
+    expected = <<~EOM
       .F
 
       Finished in 0.00
@@ -318,8 +327,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
   end
 
   def setup_basic_tu
-    @tu =
-    Class.new FakeNamedTest do
+    @tu = Class.new FakeNamedTest do
       def test_something
         assert true
       end
@@ -337,7 +345,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
   def test_run_failing_filtered
     setup_basic_tu
 
-    expected = clean <<-EOM
+    expected = <<~EOM
       .
 
       Finished in 0.00
@@ -356,14 +364,14 @@ class TestMinitestRunner < MetaMetaMetaTestCase
         assert a
       end
     end
-    Object.const_set(:Alpha, alpha)
+    Object.const_set :Alpha, alpha
 
     beta = Class.new FakeNamedTest do
       define_method :test_something do
         assert true
       end
     end
-    Object.const_set(:Beta, beta)
+    Object.const_set :Beta, beta
 
     @tus = [alpha, beta]
 
@@ -374,7 +382,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
   end
 
   def test_run_filtered_including_suite_name
-    expected = clean <<-EOM
+    expected = <<~EOM
       .
 
       Finished in 0.00
@@ -386,7 +394,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
   end
 
   def test_run_filtered_including_suite_name_string
-    expected = clean <<-EOM
+    expected = <<~EOM
       .
 
       Finished in 0.00
@@ -398,7 +406,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
   end
 
   def test_run_filtered_string_method_only
-    expected = clean <<-EOM
+    expected = <<~EOM
       ..
 
       Finished in 0.00
@@ -412,7 +420,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
   def test_run_failing_excluded
     setup_basic_tu
 
-    expected = clean <<-EOM
+    expected = <<~EOM
       .
 
       Finished in 0.00
@@ -424,7 +432,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
   end
 
   def test_run_filtered_excluding_suite_name
-    expected = clean <<-EOM
+    expected = <<~EOM
       .
 
       Finished in 0.00
@@ -436,7 +444,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
   end
 
   def test_run_filtered_excluding_suite_name_string
-    expected = clean <<-EOM
+    expected = <<~EOM
       .
 
       Finished in 0.00
@@ -448,7 +456,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
   end
 
   def test_run_filtered_excluding_string_method_only
-    expected = clean <<-EOM
+    expected = <<~EOM
 
 
       Finished in 0.00
@@ -460,14 +468,13 @@ class TestMinitestRunner < MetaMetaMetaTestCase
   end
 
   def test_run_passing
-    @tu =
-    Class.new FakeNamedTest do
+    @tu = Class.new FakeNamedTest do
       def test_something
         assert true
       end
     end
 
-    expected = clean <<-EOM
+    expected = <<~EOM
       .
 
       Finished in 0.00
@@ -479,8 +486,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
   end
 
   def test_run_skip
-    @tu =
-    Class.new FakeNamedTest do
+    @tu = Class.new FakeNamedTest do
       def test_something
         assert true
       end
@@ -490,7 +496,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
       end
     end
 
-    expected = clean <<-EOM
+    expected = <<~EOM
       .S
 
       Finished in 0.00
@@ -506,8 +512,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
   end
 
   def test_run_skip_verbose
-    @tu =
-    Class.new FakeNamedTest do
+    @tu = Class.new FakeNamedTest do
       def test_something
         assert true
       end
@@ -517,7 +522,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
       end
     end
 
-    expected = clean <<-EOM
+    expected = <<~EOM
       FakeNamedTestXX#test_something = 0.00 s = .
       FakeNamedTestXX#test_skip = 0.00 s = S
 
@@ -534,8 +539,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
   end
 
   def test_run_skip_show_skips
-    @tu =
-    Class.new FakeNamedTest do
+    @tu = Class.new FakeNamedTest do
       def test_something
         assert true
       end
@@ -545,7 +549,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
       end
     end
 
-    expected = clean <<-EOM
+    expected = <<~EOM
       .S
 
       Finished in 0.00
@@ -561,8 +565,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
   end
 
   def test_run_with_other_runner
-    @tu =
-    Class.new FakeNamedTest do
+    @tu = Class.new FakeNamedTest do
       def self.run reporter, options = {}
         @reporter = reporter
         before_my_suite
@@ -585,7 +588,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
       end
     end
 
-    expected = clean <<-EOM
+    expected = <<~EOM
       Running wacky! tests
       ..
 
@@ -634,8 +637,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
       main_latch.release
     }
 
-    @tu =
-    Class.new FakeNamedTest do
+    @tu = Class.new FakeNamedTest do
       parallelize_me!
 
       test_count.times do |i|
@@ -652,7 +654,7 @@ class TestMinitestRunner < MetaMetaMetaTestCase
       end
     end
 
-    expected = clean <<-EOM
+    expected = <<~EOM
       ..
 
       Finished in 0.00
@@ -660,20 +662,20 @@ class TestMinitestRunner < MetaMetaMetaTestCase
       2 runs, 2 assertions, 0 failures, 0 errors, 0 skips
     EOM
 
-    skip if Minitest.parallel_executor.size < 2 # locks up test runner if 1 CPU
+    skip unless Minitest.parallel_executor # locks up test runner if 1 CPU
 
-    assert_report(expected) do |reporter|
-      reporter.extend(Module.new {
-        define_method("record") do |result|
+    assert_report expected do |reporter|
+      reporter.extend Module.new {
+        define_method :record do |result|
           super(result)
           wait_latch.release
         end
 
-        define_method("report") do
+        define_method :report do
           wait_latch.await
           super()
         end
-      })
+      }
     end
     assert thread.join
   end
@@ -684,8 +686,8 @@ class TestMinitestUnitOrder < MetaMetaMetaTestCase
 
   def test_before_setup
     call_order = []
-    @tu =
-    Class.new FakeNamedTest do
+
+    @tu = Class.new FakeNamedTest do
       define_method :setup do
         super()
         call_order << :setup
@@ -700,14 +702,13 @@ class TestMinitestUnitOrder < MetaMetaMetaTestCase
 
     run_tu_with_fresh_reporter
 
-    expected = [:before_setup, :setup]
+    expected = %i[before_setup setup]
     assert_equal expected, call_order
   end
 
   def test_after_teardown
     call_order = []
-    @tu =
-    Class.new FakeNamedTest do
+    @tu = Class.new FakeNamedTest do
       define_method :teardown do
         super()
         call_order << :teardown
@@ -722,14 +723,14 @@ class TestMinitestUnitOrder < MetaMetaMetaTestCase
 
     run_tu_with_fresh_reporter
 
-    expected = [:teardown, :after_teardown]
+    expected = %i[teardown after_teardown]
     assert_equal expected, call_order
   end
 
   def test_all_teardowns_are_guaranteed_to_run
     call_order = []
-    @tu =
-    Class.new FakeNamedTest do
+
+    @tu = Class.new FakeNamedTest do
       define_method :after_teardown do
         super()
         call_order << :after_teardown
@@ -753,7 +754,7 @@ class TestMinitestUnitOrder < MetaMetaMetaTestCase
 
     run_tu_with_fresh_reporter
 
-    expected = [:before_teardown, :teardown, :after_teardown]
+    expected = %i[before_teardown teardown after_teardown]
     assert_equal expected, call_order
   end
 
@@ -780,7 +781,7 @@ class TestMinitestUnitOrder < MetaMetaMetaTestCase
     run_tu_with_fresh_reporter
 
     # Once for the parent class, once for the child
-    expected = [:setup_method, :test, :teardown_method] * 2
+    expected = %i[setup_method test teardown_method] * 2
 
     assert_equal expected, call_order
   end
@@ -813,6 +814,7 @@ class TestMinitestRunnable < Minitest::Test
     new_tc = Marshal.load Marshal.dump @tc
 
     ivars = new_tc.instance_variables.map(&:to_s).sort
+    ivars.delete "@gc_stats" # only needed if running w/ minitest-gcstats
     assert_equal expected_ivars, ivars
     assert_equal "whatever",     new_tc.name
     assert_equal 42,             new_tc.assertions
@@ -876,11 +878,11 @@ class TestMinitestRunnable < Minitest::Test
   end
 
   def test_spec_marshal_with_exception_nameerror
-    klass = describe("whatever") {
-      it("raises nameerror") {
+    klass = describe "whatever" do
+      it "raises NameError" do
         NOPE::does_not_exist
-      }
-    }
+      end
+    end
 
     rm = klass.runnable_methods.first
 
@@ -944,10 +946,9 @@ class TestMinitestRunnable < Minitest::Test
   end
 
   def test_spec_marshal_with_exception__worse_error_typeerror
-    worse_error_klass = Class.new(StandardError) do
-      # problem #1: anonymous subclass can'tmarshal, fails sanitize_exception
-      def initialize(record = nil)
-
+    worse_error_klass = Class.new StandardError do
+      # problem #1: anonymous subclass can't marshal, fails sanitize_exception
+      def initialize record = nil
         super(record.first)
       end
     end
@@ -1004,8 +1005,6 @@ class TestMinitestUnitTestCase < Minitest::Test
   # which is not threadsafe. Nearly every method in here is an
   # assertion test so it isn't worth splitting it out further.
 
-  RUBY18 = !defined? Encoding
-
   def setup
     super
 
@@ -1030,7 +1029,7 @@ class TestMinitestUnitTestCase < Minitest::Test
     $VERBOSE = orig_verbose
   end
 
-  def sample_test_case(rand)
+  def sample_test_case rand
     srand rand
     Class.new FakeNamedTest do
       100.times do |i|
@@ -1089,21 +1088,21 @@ class TestMinitestUnitTestCase < Minitest::Test
 
   def test_autorun_does_not_affect_fork_success_status
     @assertion_count = 0
-    skip "windows doesn't have fork" unless Process.respond_to?(:fork)
+    skip "windows doesn't have fork" unless Process.respond_to? :fork
     Process.waitpid(fork {})
     assert_equal true, $?.success?
   end
 
   def test_autorun_does_not_affect_fork_exit_status
     @assertion_count = 0
-    skip "windows doesn't have fork" unless Process.respond_to?(:fork)
+    skip "windows doesn't have fork" unless Process.respond_to? :fork
     Process.waitpid(fork { exit 42 })
     assert_equal 42, $?.exitstatus
   end
 
   def test_autorun_optionally_can_affect_fork_exit_status
     @assertion_count = 0
-    skip "windows doesn't have fork" unless Process.respond_to?(:fork)
+    skip "windows doesn't have fork" unless Process.respond_to? :fork
     Minitest.allow_fork = true
     Process.waitpid(fork { exit 42 })
     refute_equal 42, $?.exitstatus
@@ -1126,19 +1125,19 @@ class TestMinitestGuard < Minitest::Test
   end
 
   def test_rubinius_eh
-    assert_output "", /DEPRECATED/ do
+    assert_deprecation do
       assert self.class.rubinius? "rbx"
     end
-    assert_output "", /DEPRECATED/ do
+    assert_deprecation do
       assert self.rubinius? "rbx"
     end
   end
 
   def test_maglev_eh
-    assert_output "", /DEPRECATED/ do
+    assert_deprecation do
       assert self.class.maglev? "maglev"
     end
-    assert_output "", /DEPRECATED/ do
+    assert_deprecation do
       assert self.maglev? "maglev"
     end
   end
@@ -1179,7 +1178,7 @@ class TestMinitestUnitRecording < MetaMetaMetaTestCase
     bogus_reporter = Class.new do      # doesn't subclass AbstractReporter
       def start; @success = false; end
       # def prerecord klass, name; end # doesn't define full API
-      def record result; @success = true; end
+      def record _result; @success = true; end
       def report; end
       def passed?; end
       def results; end
@@ -1257,17 +1256,17 @@ class TestMinitestUnitRecording < MetaMetaMetaTestCase
 
     run_tu_with_fresh_reporter
 
-    exp = clean "
+    exp = <<~EOM
       Error:
       FakeNamedTestXX#test_method:
       AnError: AnError
-          FILE:LINE:in `test_method'
+          FILE:LINE:in 'test_method'
 
       Error:
       FakeNamedTestXX#test_method:
       RuntimeError: unhandled exception
-          FILE:LINE:in `teardown'
-    "
+          FILE:LINE:in 'teardown'
+    EOM
 
     assert_equal exp.strip, normalize_output(first_reporter.results.first.to_s).strip
   end
